package de.leongeorgi.pretixinfo

import com.fasterxml.jackson.annotation.JsonIgnore
import com.fasterxml.jackson.annotation.JsonProperty
import com.fasterxml.jackson.databind.JsonNode

data class Data(val event: Event) {
    companion object {
        val DEFAULT = Data(Event.DEFAULT)
    }
}

data class Event(
    val name: String,
    val slug: String,
    val organizer: Organizer,
    val categories: List<Category>,
    val items: List<Item>,
    val questions: List<Question>,
    val orders: List<Order>,
    val quotas: List<JsonNode>
) {
    val uncancelledOrders = orders.filter { it.status != OrderStatus.C }
    val orderedItems = uncancelledOrders.flatMap { it.positions.map { it.findItem(this) } }

    @JsonIgnore
    val answeredQuestions: Map<Question, List<Answer>> =
        uncancelledOrders.flatMap {
            it.positions.flatMap {
                it.answers.map { it.question to it }
            }
        }.groupBy() { it.first }.mapValues { it.value.map { it.second } }

    fun answeredQuestionsForItems(items: Map<Item, Boolean>): Map<Question, List<Answer>> = uncancelledOrders.flatMap {
        it.positions
    }.filter {
        items[it.item] ?: true
    }.flatMap {
        it.answers.map { it.question to it }
    }.groupBy { it.first }.mapValues { it.value.map { it.second } }


    val Item.category: Category get() = findCategory(this@Event)
    val Position.item: Item get() = findItem(this@Event)
    val Position.variation: Variation? get() = findVariation(this@Event)
    val Answer.question: Question get() = findQuestion(this@Event)
    val Position.parentPosition: Position? get() = findParentPosition(this@Event)
    val Position.addons: List<Position> get() = findAddons(this@Event)
    val Position.parentPositionOrSelf: Position get() = findParentPosition(this@Event) ?: this

    fun findPosition(secret: String): Pair<Position, Position>? = uncancelledOrders.asSequence()
        .flatMap { it.positions.asSequence() }
        .find { it.secret == secret }?.let { it to it.parentPositionOrSelf }

    companion object {
        val DEFAULT = Event(
            "",
            "",
            Organizer.DEFAULT,
            emptyList(),
            emptyList(),
            emptyList(),
            emptyList(),
            emptyList()
        )
    }
}

data class Order(
    val code: String,
    val status: OrderStatus,
    val user: String,
    val datetime: String,
    val fees: List<JsonNode>,
    val total: Double,
    val positions: List<Position>
) {
    companion object {
        val DEFAULT by lazy {
            Order(
                "",
                OrderStatus.N,
                "",
                "",
                emptyList(),
                0.0,
                emptyList()
            )
        }
    }

    @JsonIgnore
    private val searchableStrings = listOfNotNull(
        code,
        user
    )

    fun matches(search: String): Pair<Boolean, Position?> {
        if (search.isEmpty()) {
            return true to null
        }

        if (searchableStrings.any { it.contains(search, true) }) {
            return true to null
        }

        val matchingPositions = positions.filter { it.matches(search) }

        return if (matchingPositions.isNotEmpty()) {
            true to matchingPositions.singleOrNull()
        } else {
            false to null
        }
    }
}

data class Position(
    val id: Int,
    @JsonProperty("item") val itemId: Int,
    @JsonProperty("variation") val variationId: Int?,
    val price: Double,
    @JsonProperty("attendee_name") val attendeeName: String?,
    @JsonProperty("attendee_email") val attendeeEmail: String?,
    val secret: String,
    @JsonProperty("addon_to") val addonTo: Int?,
    val answers: List<Answer>
) {
    fun findItem(event: Event) = event.items.first { it.id == itemId }

    fun findVariation(event: Event): Variation? {
        return findItem(event).variations.find { it.id == (variationId ?: return null) }
    }

    @JsonIgnore
    val searchableStrings = listOfNotNull(
        *answers.map { it.answer }.toTypedArray(),
        attendeeEmail,
        attendeeName,
        secret
    )

    @JsonIgnore
    val isAddon: Boolean = addonTo != null

    fun findAddons(event: Event): List<Position> {
        return event.uncancelledOrders.flatMap { it.positions }.filter { it.addonTo == id }
    }

    fun matches(search: String): Boolean {
        if (search.isEmpty()) {
            return true
        }
        return searchableStrings.any { it.contains(search, ignoreCase = true) }
    }


    fun findParentPosition(event: Event): Position? {
        return event.uncancelledOrders.flatMap { it.positions }.find { it.id == addonTo }
    }

    companion object {
        val DEFAULT = Position(
            0,
            0,
            0,
            0.0,
            null,
            null,
            "",
            null,
            emptyList()
        )
    }
}

data class Answer(
    @JsonProperty("question") val questionId: Int,
    val answer: String
) {
    fun findQuestion(event: Event) = event.questions.first { it.id == questionId }
}

enum class OrderStatus {
    @JsonProperty("n")
    N,
    @JsonProperty("c")
    C,
    @JsonProperty("p")
    PAID
}

data class Question(
    val id: Int,
    val question: String,
    val type: QuestionType
)

enum class QuestionType {
    B, C, M, S, T
}

data class Category(
    val id: Int,
    val name: String,
    @JsonProperty("internal_name") val internalName: String?
)

data class Item(
    val id: Int,
    val name: String,
    @JsonProperty("internal_name") val internalName: String?,
    @JsonProperty("category") val categoryId: Int,
    val price: Double,
    @JsonProperty("tax_rate") val taxRate: Double,
    @JsonProperty("tax_name") val taxName: String?,
    val admission: Boolean,
    val active: Boolean,
    val variations: List<Variation>
) {
    fun findCategory(event: Event) = event.categories.first { it.id == categoryId }
}

data class Variation(
    val id: Int,
    val active: Boolean,
    val price: Double,
    val name: String
)

data class Organizer(
    val name: String,
    val slug: String
) {
    companion object {

        val DEFAULT = Organizer("", "")
    }
}